@@ -25,14 +25,16 @@ struct NodeInfo {
 /// Metrics tracking for remote nodes to help
 /// choose which remote node to initiate the next round with.
 pub(super) struct Metrics {
-    /// Map of remote nodes.
-    map: HashMap<StateKey, NodeInfo>,
+    /// Map of remote agents.
+    map: HashMap<Arc<KitsuneAgent>, NodeInfo>,
+
     // Number of times we need to force initiate
     // the next round.
     force_initiates: u8,
 }
 
 /// Outcome of a gossip round.
+#[derive(PartialOrd, Ord, PartialEq, Eq)]
 pub(super) enum RoundOutcome {
     Success(Instant),
     Error(Instant),
@@ -45,34 +47,48 @@ impl Metrics {
     }
 
     /// Record a gossip round has been initiated by us.
-    pub(super) fn record_initiate(&mut self, key: StateKey) {
-        let info = self.map.entry(key).or_default();
-        record_instant(&mut info.initiates);
-        info.current_round = true;
+    pub(super) fn record_initiate(&mut self, remote_agent_list: &[AgentInfoSigned]) {
+        for agent_info in remote_agent_list {
+            let info = self.map.entry(agent_info.agent.clone()).or_default();
+            record_instant(&mut info.initiates);
+            info.current_round = true;
+        }
     }
 
     /// Record a remote gossip round has started.
-    pub(super) fn record_remote_round(&mut self, key: StateKey) {
-        let info = self.map.entry(key).or_default();
-        record_instant(&mut info.remote_rounds);
-        info.current_round = true;
+    pub(super) fn record_remote_round(&mut self, remote_agent_list: &[AgentInfoSigned]) {
+        for agent_info in remote_agent_list {
+            let info = self.map.entry(agent_info.agent.clone()).or_default();
+            record_instant(&mut info.remote_rounds);
+            info.current_round = true;
+        }
     }
 
     /// Record a gossip round has completed successfully.
-    pub(super) fn record_success(&mut self, key: StateKey) {
-        let info = self.map.entry(key).or_default();
-        record_instant(&mut info.complete_rounds);
-        info.current_round = false;
-        if info.is_initiate_round() {
+    pub(super) fn record_success(&mut self, remote_agent_list: &[AgentInfoSigned]) {
+        let mut should_dec_force_initiates = false;
+
+        for agent_info in remote_agent_list {
+            let info = self.map.entry(agent_info.agent.clone()).or_default();
+            record_instant(&mut info.complete_rounds);
+            info.current_round = false;
+            if info.is_initiate_round() {
+                should_dec_force_initiates = true;
+            }
+        }
+
+        if should_dec_force_initiates {
             self.force_initiates = self.force_initiates.saturating_sub(1);
         }
     }
 
     /// Record a gossip round has finished with an error.
-    pub(super) fn record_error(&mut self, key: StateKey) {
-        let info = self.map.entry(key).or_default();
-        record_instant(&mut info.errors);
-        info.current_round = false;
+    pub(super) fn record_error(&mut self, remote_agent_list: &[AgentInfoSigned]) {
+        for agent_info in remote_agent_list {
+            let info = self.map.entry(agent_info.agent.clone()).or_default();
+            record_instant(&mut info.errors);
+            info.current_round = false;
+        }
     }
 
     /// Record that we should force initiate the next few rounds.
@@ -81,33 +97,43 @@ impl Metrics {
     }
 
     /// Get the last successful round time.
-    pub(super) fn last_success(&self, key: &StateKey) -> Option<&Instant> {
-        self.map
-            .get(key)
-            .and_then(|info| info.complete_rounds.back())
+    pub(super) fn last_success(&self, remote_agent_list: &[AgentInfoSigned]) -> Option<&Instant> {
+        remote_agent_list
+            .iter()
+            .filter_map(|agent_info| self.map.get(&agent_info.agent))
+            .map(|info| info.complete_rounds.back())
+            .flatten()
+            .min()
     }
 
     /// Is this node currently in an active round?
-    pub(super) fn is_current_round(&self, key: &StateKey) -> bool {
-        self.map.get(key).map_or(false, |info| info.current_round)
+    pub(super) fn is_current_round(&self, remote_agent_list: &[AgentInfoSigned]) -> bool {
+        remote_agent_list
+            .iter()
+            .filter_map(|agent_info| self.map.get(&agent_info.agent))
+            .map(|info| info.current_round)
+            .any(|x| x)
     }
 
-    /// What was the last outcome for this nodes gossip round?
-    pub(super) fn last_outcome(&self, key: &StateKey) -> Option<RoundOutcome> {
-        self.map.get(key).and_then(
-            |info| match (info.errors.back(), info.complete_rounds.back()) {
-                (Some(error), Some(success)) => {
-                    if error > success {
-                        Some(RoundOutcome::Error(*error))
-                    } else {
-                        Some(RoundOutcome::Success(*success))
-                    }
-                }
-                (Some(error), None) => Some(RoundOutcome::Error(*error)),
-                (None, Some(success)) => Some(RoundOutcome::Success(*success)),
-                (None, None) => None,
-            },
-        )
+    /// What was the last outcome for this node's gossip round?
+    pub(super) fn last_outcome(
+        &self,
+        remote_agent_list: &[AgentInfoSigned],
+    ) -> Option<RoundOutcome> {
+        remote_agent_list
+            .iter()
+            .filter_map(|agent_info| self.map.get(&agent_info.agent))
+            .map(|info| {
+                [
+                    info.errors.back().map(|x| RoundOutcome::Error(*x)),
+                    info.complete_rounds
+                        .back()
+                        .map(|x| RoundOutcome::Success(*x)),
+                ]
+            })
+            .flatten()
+            .flatten()
+            .max()
     }
 
     /// Should we force initiate the next round?
