@@ -6,9 +6,10 @@ use super::*;
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 /// A remote node we can connect to.
 /// Note that a node can contain many agents.
-struct Node {
-    target: GossipTgt,
-    url: TxUrl,
+pub(crate) struct Node {
+    pub(crate) agent_info_list: Vec<AgentInfoSigned>,
+    pub(crate) cert: Tx2Cert,
+    pub(crate) url: TxUrl,
 }
 
 impl ShardedGossipLocal {
@@ -17,7 +18,7 @@ impl ShardedGossipLocal {
         &self,
         arc_set: Arc<DhtArcSet>,
         local_agents: &HashSet<Arc<KitsuneAgent>>,
-    ) -> KitsuneResult<Option<(GossipTgt, TxUrl)>> {
+    ) -> KitsuneResult<Option<Node>> {
         let mut remote_nodes: HashMap<Tx2Cert, Node> = HashMap::new();
 
         // Get all the remote nodes in this arc set.
@@ -52,7 +53,7 @@ impl ShardedGossipLocal {
                         .ok()
                         .map(|purl| {
                             (
-                                info.agent.clone(),
+                                info.clone(),
                                 Tx2Cert::from(purl.digest()),
                                 TxUrl::from(url.as_str()),
                             )
@@ -64,19 +65,17 @@ impl ShardedGossipLocal {
 
             // If we found a remote address add this agent to the node
             // or create the node if it doesn't exist.
-            if let Some((agent, cert, url)) = info {
+            if let Some((info, cert, url)) = info {
                 match remote_nodes.get_mut(&cert) {
                     // Add the agent to the node.
-                    Some(node) => node.target.agents.push(agent),
+                    Some(node) => node.agent_info_list.push(info),
                     None => {
                         // This is a new node.
                         remote_nodes.insert(
                             cert.clone(),
                             Node {
-                                target: GossipTgt {
-                                    agents: vec![agent],
-                                    cert,
-                                },
+                                agent_info_list: vec![info],
+                                cert,
                                 url,
                             },
                         );
@@ -90,8 +89,7 @@ impl ShardedGossipLocal {
         // We could clone the metrics store out of the lock here but I don't think
         // the next_remote_node will be that slow so we can just choose the next node inline.
         self.inner.share_mut(|i, _| {
-            let node = next_remote_node(remote_nodes, &i.metrics, tuning_params)
-                .map(|n| (n.target, n.url));
+            let node = next_remote_node(remote_nodes, &i.metrics, tuning_params);
             Ok(node)
         })
     }
@@ -113,8 +111,8 @@ fn next_remote_node(
     // Note the smaller an Instant the longer it is in the past.
     remote_nodes.sort_unstable_by(|a, b| {
         match (
-            metrics.last_success(a.cert()),
-            metrics.last_success(b.cert()),
+            metrics.last_success(&a.agent_info_list),
+            metrics.last_success(&b.agent_info_list),
         ) {
             // Choose the smallest (oldest) Instant.
             (Some(a), Some(b)) => a.cmp(b),
@@ -136,9 +134,9 @@ fn next_remote_node(
     remote_nodes
         .into_iter()
         // Don't initiate with nodes we are currently gossiping with.
-        .filter(|n| !metrics.is_current_round(n.cert()))
+        .filter(|n| !metrics.is_current_round(&n.agent_info_list))
         .find(|n| {
-            match metrics.last_outcome(n.cert()) {
+            match metrics.last_outcome(&n.agent_info_list) {
                 Some(metrics::RoundOutcome::Success(when)) => {
                     // If we should force initiate then we don't need to wait for the delay.
                     metrics.forced_initiate()
@@ -154,13 +152,6 @@ fn next_remote_node(
         })
 }
 
-impl Node {
-    /// Get the cert for this node.
-    fn cert(&self) -> &Tx2Cert {
-        self.target.cert()
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use crate::gossip::sharded_gossip::metrics::Metrics;
@@ -186,6 +177,25 @@ mod tests {
         )
     }
 
+    /// Generate a random pseudo-valid signed agent info
+    fn random_agent_info(rng: &mut ThreadRng) -> AgentInfoSigned {
+        let space = Arc::new(KitsuneSpace(vec![0x01; 36]));
+        let mut agent = vec![0x00; 36];
+        rng.fill(&mut agent[..]);
+        let agent = Arc::new(KitsuneAgent(agent));
+
+        futures::executor::block_on(AgentInfoSigned::sign(
+            space,
+            agent,
+            42,
+            vec![random_url(rng).into()],
+            42,
+            69,
+            |_| async move { Ok(Arc::new(vec![0x03; 64].into())) },
+        ))
+        .unwrap()
+    }
+
     /// Tuning params with no delay on recently gossiped to nodes.
     fn tuning_params_no_delay() -> KitsuneP2pTuningParams {
         let mut t = tuning_params_struct::KitsuneP2pTuningParams::default();
@@ -206,11 +216,15 @@ mod tests {
         let mut rng = thread_rng();
         (0..n)
             .map(|_| {
-                let url = random_url(&mut rng);
+                let info = random_agent_info(&mut rng);
+                let url = info.url_list.get(0).unwrap().clone();
                 let url = TxUrl::from(url.as_str());
                 let purl = kitsune_p2p_proxy::ProxyUrl::from_full(url.as_str()).unwrap();
-                let target = GossipTgt::new(vec![], Tx2Cert::from(purl.digest()));
-                Node { target, url }
+                Node {
+                    agent_info_list: vec![info],
+                    cert: Tx2Cert::from(purl.digest()),
+                    url,
+                }
             })
             .collect()
     }
@@ -246,17 +260,15 @@ mod tests {
 
         // - Pop the last node off the list.
         let last = remote_nodes.pop().unwrap();
-        let cert = last.cert().clone();
 
         // - Record a successful initiate round for the last node at the earliest time.
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        metrics.record_initiate(&last.agent_info_list);
+        metrics.record_success(&last.agent_info_list);
 
         // - Record successful initiate rounds for the rest of the nodes at later times.
         for node in remote_nodes.iter() {
-            let cert = node.cert().clone();
-            metrics.record_initiate(cert.clone());
-            metrics.record_success(cert);
+            metrics.record_initiate(&node.agent_info_list);
+            metrics.record_success(&node.agent_info_list);
         }
 
         // - Push the last node back into the remote nodes.
@@ -285,9 +297,8 @@ mod tests {
 
         // - Record successful initiate rounds for the rest of the nodes.
         for node in remote_nodes.iter() {
-            let cert = node.cert().clone();
-            metrics.record_initiate(cert.clone());
-            metrics.record_success(cert);
+            metrics.record_initiate(&node.agent_info_list);
+            metrics.record_success(&node.agent_info_list);
         }
 
         // - Push the last node back into the remote nodes.
@@ -314,9 +325,8 @@ mod tests {
 
         // - Record successful initiate rounds for the rest of the nodes.
         for node in remote_nodes.iter() {
-            let cert = node.cert().clone();
-            metrics.record_initiate(cert.clone());
-            metrics.record_success(cert);
+            metrics.record_initiate(&node.agent_info_list);
+            metrics.record_success(&node.agent_info_list);
         }
 
         // - Push the last two nodes back into the remote nodes.
@@ -355,8 +365,7 @@ mod tests {
         // - Record remote rounds for the rest of the nodes
         // but don't record any successes.
         for node in remote_nodes.iter() {
-            let cert = node.cert().clone();
-            metrics.record_remote_round(cert);
+            metrics.record_remote_round(&node.agent_info_list);
         }
 
         let r = next_remote_node(remote_nodes.clone(), &metrics, tuning_params_no_delay());
@@ -365,9 +374,8 @@ mod tests {
         assert!(r.is_none());
 
         // - Record the last node as a successful round and push it into the list.
-        let cert = last.cert().clone();
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        metrics.record_initiate(&last.agent_info_list);
+        metrics.record_success(&last.agent_info_list);
         remote_nodes.push(last);
 
         let r = next_remote_node(remote_nodes.clone(), &metrics, tuning_params_no_delay());
@@ -387,9 +395,8 @@ mod tests {
 
         // - Record successful initiate rounds for the all of the nodes.
         for node in remote_nodes.iter() {
-            let cert = node.cert().clone();
-            metrics.record_initiate(cert.clone());
-            metrics.record_success(cert);
+            metrics.record_initiate(&node.agent_info_list);
+            metrics.record_success(&node.agent_info_list);
         }
 
         let r = next_remote_node(
@@ -427,9 +434,8 @@ mod tests {
 
         // - Record error outcomes for every node.
         for node in remote_nodes.iter() {
-            let cert = node.cert().clone();
-            metrics.record_initiate(cert.clone());
-            metrics.record_error(cert);
+            metrics.record_initiate(&node.agent_info_list);
+            metrics.record_error(&node.agent_info_list);
         }
 
         let r = next_remote_node(
@@ -480,17 +486,15 @@ mod tests {
 
         // - Pop the last node off the list.
         let last = remote_nodes.pop().unwrap();
-        let cert = last.cert().clone();
 
         // - Record a successful initiate round for the last node before the other nodes.
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        metrics.record_initiate(&last.agent_info_list);
+        metrics.record_success(&last.agent_info_list);
 
         // - Record successful initiate rounds for the rest of the nodes.
         for node in remote_nodes.iter() {
-            let cert = node.cert().clone();
-            metrics.record_initiate(cert.clone());
-            metrics.record_success(cert);
+            metrics.record_initiate(&node.agent_info_list);
+            metrics.record_success(&node.agent_info_list);
         }
 
         // - Push the last node back on the list.
@@ -521,9 +525,9 @@ mod tests {
         assert_eq!(r, remote_nodes.last().cloned());
 
         // - Record this successful initiate round.
-        let cert = remote_nodes.last().unwrap().cert().clone();
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        let last = remote_nodes.last().unwrap();
+        metrics.record_initiate(&last.agent_info_list);
+        metrics.record_success(&last.agent_info_list);
 
         let r = next_remote_node(
             remote_nodes.clone(),
@@ -536,9 +540,9 @@ mod tests {
         assert_eq!(r, remote_nodes.first().cloned());
 
         // - Record this successful initiate round.
-        let cert = remote_nodes.first().unwrap().cert().clone();
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        let first = remote_nodes.first().unwrap();
+        metrics.record_initiate(&first.agent_info_list);
+        metrics.record_success(&first.agent_info_list);
 
         let r = next_remote_node(
             remote_nodes.clone(),
@@ -583,9 +587,8 @@ mod tests {
 
         // - Record the successful initiate round for this node.
         let node = expected_node(1).unwrap();
-        let cert = node.cert().clone();
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        metrics.record_initiate(&node.agent_info_list);
+        metrics.record_success(&node.agent_info_list);
 
         let r = next_remote_node(
             remote_nodes.clone(),
@@ -599,9 +602,8 @@ mod tests {
 
         // - Record the successful initiate round for this node.
         let node = expected_node(2).unwrap();
-        let cert = node.cert().clone();
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        metrics.record_initiate(&node.agent_info_list);
+        metrics.record_success(&node.agent_info_list);
 
         let r = next_remote_node(
             remote_nodes.clone(),
@@ -628,9 +630,8 @@ mod tests {
 
         // - Record the successful initiate round for this node.
         let node = expected_node(3).unwrap();
-        let cert = node.cert().clone();
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        metrics.record_initiate(&node.agent_info_list);
+        metrics.record_success(&node.agent_info_list);
 
         // - Forth force initiate overlaps with third so it resets.
         metrics.record_force_initiate();
@@ -647,9 +648,8 @@ mod tests {
 
         // - Record the successful initiate round for this node.
         let node = expected_node(4).unwrap();
-        let cert = node.cert().clone();
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        metrics.record_initiate(&node.agent_info_list);
+        metrics.record_success(&node.agent_info_list);
 
         let r = next_remote_node(
             remote_nodes.clone(),
@@ -663,9 +663,8 @@ mod tests {
 
         // - Record the successful initiate round for this node.
         let node = expected_node(5).unwrap();
-        let cert = node.cert().clone();
-        metrics.record_initiate(cert.clone());
-        metrics.record_success(cert);
+        metrics.record_initiate(&node.agent_info_list);
+        metrics.record_success(&node.agent_info_list);
 
         let r = next_remote_node(
             remote_nodes.clone(),
