@@ -92,34 +92,3 @@ impl GossipModuleFactory {
             .spawn_gossip_task(tuning_params, space, ep_hnd, evt_sender)
     }
 }
-
-/// The specific provenance/destination of gossip is a particular Agent on
-/// a connection specified by a Tx2Cert
-#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, derive_more::Constructor)]
-pub struct GossipTgt {
-    /// The agents on the remote node for whom this gossip is intended.
-    /// In the current full-sync case, it makes sense to address gossip to all
-    /// known agents on a node, but after sharding, we may make this a single
-    /// agent target.
-    pub agents: Vec<Arc<KitsuneAgent>>,
-    /// The cert which represents the remote node to talk to.
-    pub cert: Tx2Cert,
-}
-
-impl GossipTgt {
-    /// Accessor
-    pub fn agents(&self) -> &Vec<Arc<KitsuneAgent>> {
-        &self.agents
-    }
-
-    /// Accessor
-    pub fn cert(&self) -> &Tx2Cert {
-        self.as_ref()
-    }
-}
-
-impl AsRef<Tx2Cert> for GossipTgt {
-    fn as_ref(&self) -> &Tx2Cert {
-        &self.cert
-    }
-}
